# -*- coding: utf-8 -*-
import pandas as pd

def normalize(df, style = 'mean'):
    if style == 'mean':
        df_mean,df_std = df.mean(),df.std()
        return (df-df_mean)/df_std
    elif style == 'minmax':
        col_min,col_max = df.min(),df.max()
        return (df-col_min)/(col_max-col_min)

def col_normalize(df,col_name, style = 'mean'):
    return normalize(df[col_name],style)

def row_normalize(df,row_name,style = 'mean'):
    return normalize(df.loc[row_name,:],style)

def norms(df, col_names = None,row_names = None,style = 'mean', as_group = False, axis = 0):
    if col_names is None:
        if row_names is not None:
            df = df.loc[row_names,:]
    else:
        if row_names is None:
            df = df.loc[:,col_names]
        else:
            df = df.loc[row_names,col_names]
    if as_group:
        return normalize(df,style)
    if axis == 0:
        return pd.concat([col_normalize(df,col_name,style) for col_name in df.columns],axis = 1)
    elif axis == 1:
        return pd.concat([row_normalize(df,row_name,style) for row_name in df.index])
    else:
        return normalize(df,style)

def percentiles(df):
    """ Takes a dataframe and returns the quartiles for each column,
    or an error message if there are no columns with quantitative data.
    """
    assert type(df) is pd.DataFrame
    try:
        return df.quantile(q = [0,.1,.2,.3,.4,.5,.6,.7,.8,.9,1])
    except ValueError:
        return None#"No columns with numeric data."
    
def dtypes_summary(df):
    """ Takes in a dataframe and returns a dataframe with
    information on the data-types present in each column.
    """
    assert type(df) is pd.DataFrame
    output_df = pd.DataFrame([])
    row_count = df.shape[0]
    row_indexes = ['rows_numerical','rows_string','rows_date_time','category_count','largest_category','rows_na','rows_total']
    for colname in df:
        data = df[colname] # data is the pandas series associated with this column
        # number of numerical values in the column
        rows_numerical = pd.to_numeric(data,errors = 'coerce').count()
        # number of values that can't be coerced to a numerical
        rows_string = row_count - rows_numerical
        # number of values that can be coerced to a date-time object
        rows_date_time = pd.to_datetime(data,errors = 'coerce',infer_datetime_format = True).count()
        # categories in column
        value_counts = data.value_counts()
        # number of different values in the dataframe
        categories = data.value_counts().count()
        # largest category
        largest_category = value_counts[0]
        # number of null/missing values
        rows_na = data.isnull().sum()
        # build the output list
        output_data = [rows_numerical, rows_string, rows_date_time, categories, 
                       largest_category,rows_na,row_count]
        # add to dataframe
        output_df.loc[:,colname] = pd.Series(output_data)

    # row names
    output_df.index = row_indexes
    return output_df


# -*- coding: utf-8 -*-
import pandas as pd
from mc_io import preview,get_info,output_to_file,title_line,format_row
from mc_stats import percentiles,dtypes_summary
from mc_utils import rows
# Pre-cleaning

# This method takes in a DataFrame object, as well as a few parameters,
# and outputs a DataFrame that summarizes some of the possible problems
# that might have to be addressed in cleaning
def summary(df,preview_rows = 5,preview_max_cols = 0,
            memory_usage = 'deep',display_width = None,
            output_file = None, output_safe = True):
    """ Prints information about the DataFrame to a file or to the prompt.

    Parameters
    ----------
    df : DataFrame
        The DataFrame to summarize
    preview_rows : int, default 5
        Amount of rows to preview from the head and tail of the DataFrame
    preview_max_cols : int, default 0
        Maximum amount of columns to preview. Set to None to preview all
        columns, and set to 0 to preview as many as fit in the screen's width
    memory_usage : boolean or 'deep', default 'deep'
        Type of output that the 'memory usage' section of the .info() call should have
    display_width : int, default None
        Width of output. Can be width of file or width of console for printing.
        Set to None for pandas to detect it from console.
    output_file : path-like, default None
        If not None, this will be used as the path of the output file, and this
        function will print to a file instead of to the prompt
    output_safe : boolean, default True
        If True and output_file is not None, this function will not overwrite any
        existing files.
    """
    assert type(df) is pd.DataFrame

    # --------Values of data-----------
    df_preview = preview(df,preview_rows = 5,preview_max_cols = 0)
    info = get_info(df,verbose = True, max_cols = None,memory_usage = memory_usage,null_counts = True)
    percent_values = percentiles(df)
    dtypes = dtypes_summary(df).apply(format_row,args = [rows(df)],axis = 1)
    dtypes.columns = df.columns
    potential_outliers = None # TODO add 'potential outliers' output

    # ----------Build lists------------
    title_list = ['Preview','Describe','Info']
    info_list = [df_preview,df.describe().transpose(),info]
    if percent_values is not None:
        title_list.append('Percentile Details')
        info_list.append(percent_values)
    title_list+=['Missing Values Summary','Potential Outliers']
    info_list+=[dtypes,potential_outliers]
    output = zip(title_list,info_list)

    # -------Print or output-----------

    # Get initial display settings
    initial_max_cols = pd.get_option('display.max_columns')
    initial_max_rows = pd.get_option('display.max_rows')
    initial_width = pd.get_option('display.width')

    # Reformat displays
    pd.set_option('display.max_columns', None)
    pd.set_option('display.max_rows',None)
    if display_width is not None:
            pd.set_option('display.width',display_width)

    #Output information to print line or file
    if output_file is None:
         for title, value in output:
            print(title_line(title))
            print(value)
    else:
        output_to_file(df,output_file,output_safe)
    # Reset display settings
    pd.set_option('display.max_columns', initial_max_cols)
    pd.set_option('display.max_columns', initial_max_rows)
    pd.set_option('display.max_columns', initial_width)


# -*- coding: utf-8 -*-
from mc_utils import rows
from mc_funcs import colname_gen, col_scrubf,col_scrubb

def smart_scrub(df,col_name,cutoff = 1):
    scrubf = smart_scrubf(df,col_name,cutoff)
    scrubb = smart_scrubb(df,col_name,cutoff)
    return (scrubf, scrubb)

def smart_scrubf(df,col_name,cutoff = 1):
    scrubbed = ""
    while True:
        valcounts = df[col_name].str[:len(scrubbed)+1].value_counts()
        if not len(valcounts):
            break
        if not valcounts[0] >= cutoff * rows(df):
            break
        scrubbed=valcounts.index[0]
    if scrubbed == '':
        return None
    which = df[col_name].str.startswith(scrubbed)
    col_scrubf(df,col_name,which,len(scrubbed),True)
    if not which.all():
        new_col_name = colname_gen(df,"{}_sf-{}".format(col_name,scrubbed))
        df[new_col_name] = which
    return scrubbed

def smart_scrubb(df,col_name,cutoff = 1):
    scrubbed = ""
    while True:
        valcounts = df[col_name].str[-len(scrubbed)-1:].value_counts()
        if not len(valcounts):
            break
        if not valcounts[0] >= cutoff * rows(df):
            break
        scrubbed=valcounts.index[0]
    if scrubbed == '':
        return None
    which = df[col_name].str.endswith(scrubbed)
    col_scrubb(df,col_name,which,len(scrubbed),True)
    if not which.all():
        new_col_name = colname_gen(df,"{}_sb-{}".format(col_name,scrubbed))
        df[new_col_name] = which
    return scrubbed

def smart_coerce():
    pass



from setuptools import setup,find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="mr_clean",
    version="0.0.1",
    author="A1Liu",
    author_email="albertymliu@gmail.com",
    description="A small package for cleaning data.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/A1Liu/mr_clean",
    packages=find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
    ),
)



# -*- coding: utf-8 -*-
import pandas as pd










def validate(df,coerce_numeric,coerce_dt,coerce_categorical): # validates input
    assert type(df) is pd.DataFrame
    column_dict = {}
    for element in coerce_numeric + coerce_dt + coerce_categorical: # these lists must be mutually exclusive
        assert type(element) is str
        assert not element in column_dict
        column_dict[element] = True


# -*- coding: utf-8 -*-
import pandas as pd
from mc_utils import cols
from collections import deque

#None of these functions should have much logic.


# --------- Blind ops on entire table ---------------------

def colname_gen(df,col_name = 'unnamed_col'):
        if col_name not in df.keys():
            yield col_name
        id_number = 0
        while True:
            col_name = col_name + str(id_number)
            if col_name in df.keys():
                id_number+=1
            else:
                return col_name

# --------- Blind ops on entire table (destructive) ----------------

def clean_colnames(df):
    col_list = []
    for index in range(cols(df)):
        col_list.append(df.columns[index].strip().lower().replace(' ','_'))
    df.columns = col_list

def reindex(df):
    df.reset_index()

# ----- Blind ops on single columns-------------

def col_strip(df,col_name,dest = False):
    if dest:
        df[col_name] = df[col_name].str.strip()
    else:
        return df[col_name].str.strip()
    
def col_scrubf(df,col_name,which,count = 1,dest = False):
    if dest:
        df.loc[which,col_name] = df.loc[which,col_name].str[count:]
    else:
        new_col = df[col_name].copy()
        new_col[which] = df.loc[which,col_name].str[count:]
        return new_col
    
def col_scrubb(df,col_name,which, count = 1,dest = False):
    if dest:
        df.loc[which,col_name] = df.loc[which,col_name].str[:-count]
    else:
        new_col = df[col_name].copy()
        new_col[which] = df.loc[which,col_name].str[:-count]
        return new_col

def col_to_numeric(df,col_name, dest = False):
    new_col = pd.to_numeric(df[col_name], errors = 'coerce')
    if dest:
        set_col(df,col_name,new_col)
    else:
        return new_col

def col_to_dt(df,col_name,set_format = None,infer_format = True, dest = False):
    new_col = pd.to_datetime(df[col_name],errors = 'coerce',
                                    format = set_format,infer_datetime_format = infer_format)
    if dest:
        set_col(df,col_name,new_col)
    else:
        return new_col

def col_to_cat(df,col_name, dest = False):
    new_col = df[col_name].astype('category')
    if dest:
        set_col(df,col_name,new_col)
    else:
        return new_col

# ------- Blind ops on single columns (destructive) ------

def set_col(df,col_name, new_values):
    df[col_name] = new_values

def col_rename(df,col_name,new_col_name):
    col_list = list(df.columns)
    for index,value in enumerate(col_list):
        if value == col_name:
            col_list[index] = new_col_name
            break
    df.columns = col_list

def col_mod(df,col_name,func,*args,**kwargs):
    backup = df[col_name].copy()
    try:
        return_val = func(df,col_name,*args,**kwargs)
        if return_val is not None:
            set_col(df,col_name,return_val)
    except:
        df[col_name] = backup

# ------- Blind operations on multiple columns ----------
    
def cols_strip(df,col_list, dest = False):
    if not dest:
        return [col_strip(df,col_name) for col_name in col_list]
    for col_name in col_list:
        col_strip(df,col_name,dest)

def cols_to_numeric(df, col_list,dest = False):
    if not dest:
        return [col_to_numeric(df,col_name) for col_name in col_list]
    for col_name in col_list:
        col_to_numeric(df,col_name,dest)
        
def cols_to_dt(df, col_list,set_format = None,infer_format = True,dest = False):
    if not dest:
        return [col_to_dt(df,col_name,set_format,infer_format) for col_name in col_list]
    for col_name in col_list:
        col_to_dt(df,col_name,set_format,infer_format,dest)

def cols_to_cat(df, col_list,dest = False):
    # Convert a list of columns to categorical
    if not dest:
        return [col_to_cat(df,col_name) for col_name in col_list]
    for col_name in col_list:
        col_to_cat(df,col_name,dest)

def cols_(df,col_list,func,*args,**kwargs):
    #Do a function over a list of columns and return the result
    return [func(df,col_name,*args,**kwargs) for col_name in col_list]

# ------- Blind operations on multiple columns (destructive) ----------

def cols_rename(df,col_names = [], new_col_names = []):
    assert len(col_names) == len(new_col_names)
    for old_name,new_name in zip(col_names,new_col_names):
        col_rename(df,old_name,new_name)

# ------- Get formatting information ----------

def col_dtypes(df): # Does some work to reduce possibility of errors and stuff
    """ 
    Pandas datatypes are as follows:
    object,number,datetime,category,timedelta,datetimetz
    This method uses queues and iterates over the columns in linear time.
    It does extra steps to ensure that no further work with numpy datatypes needs
    to be done.
    """
    test_list = [col_isobj,col_isnum,col_isdt,col_iscat,col_istdelt,col_isdtz]
    names_list = ["object","number","datetime","category","timedelta","datetimetz"]
    deque_list = [(deque(col_method(df)),name) \
                  for col_method,name in zip(test_list,names_list) if len(col_method(df))]
    type_dict = {}
    for col_name in df.columns:
        for que,name in deque_list:
            if len(que):
                if que[0] == col_name:
                    que.popleft()
                    type_dict[col_name] = name
                    break
    return type_dict

def col_isobj(df, col_name = None):
    col_list = df.select_dtypes(include = ['object']).columns
    if col_name is None:
        return col_list
    else:
        return col_name in col_list

def col_isnum(df,col_name = None):
    col_list = df.select_dtypes(include = ['number']).columns
    if col_name is None:
        return col_list
    else:
        return col_name in col_list

def col_isdt(df,col_name = None):
    col_list = df.select_dtypes(include = ['datetime']).columns
    if col_name is None:
        return col_list
    else:
        return col_name in col_list

def col_iscat(df,col_name = None):
    col_list = df.select_dtypes(include = ['category']).columns
    if col_name is None:
        return col_list
    else:
        return col_name in col_list

def col_istdelt(df,col_name = None):
    col_list = df.select_dtypes(include = ['timedelta']).columns
    if col_name is None:
        return col_list
    else:
        return col_name in col_list

def col_isdtz(df,col_name = None):
    col_list = df.select_dtypes(include = ['datetimetz']).columns
    if col_name is None:
        return col_list
    else:
        return col_name in col_list


